@@ -1,6 +1,6 @@
 # Utilities for performing tasks related to accessibility inspection.
 #
-# Copyright 2023 Igalia, S.L.
+# Copyright 2023-2025 Igalia, S.L.
 # Author: Joanmarie Diggs <jdiggs@igalia.com>
 #
 # This library is free software; you can redistribute it and/or
@@ -18,8 +18,8 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
-# pylint: disable=broad-exception-caught
 # pylint: disable=too-many-branches
+# pylint: disable=too-many-public-methods
 # pylint: disable=too-many-return-statements
 # pylint: disable=too-many-statements
 # pylint: disable=wrong-import-position
@@ -29,20 +29,21 @@
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2023 Igalia, S.L."
+__copyright__ = "Copyright (c) 2023-2025 Igalia, S.L."
 __license__   = "LGPL"
 
 import functools
 import inspect
+import queue
 import threading
 import time
-from typing import Optional
 
 import gi
 gi.require_version("Atspi", "2.0")
 from gi.repository import Atspi
 
 from . import debug
+from .ax_component import AXComponent
 from .ax_object import AXObject
 from .ax_selection import AXSelection
 from .ax_table import AXTable
@@ -63,8 +64,6 @@ class AXUtilities:
     # Things we cache.
     SET_MEMBERS: dict[int, list[Atspi.Accessible]] = {}
     IS_LAYOUT_ONLY: dict[int, tuple[bool, str]] = {}
-    DISPLAYED_DESCRIPTION: dict[int, str] = {}
-    DISPLAYED_LABEL: dict[int, str] = {}
 
     _lock = threading.Lock()
 
@@ -94,11 +93,9 @@ class AXUtilities:
         with AXUtilities._lock:
             AXUtilities.SET_MEMBERS.clear()
             AXUtilities.IS_LAYOUT_ONLY.clear()
-            AXUtilities.DISPLAYED_DESCRIPTION.clear()
-            AXUtilities.DISPLAYED_LABEL.clear()
 
     @staticmethod
-    def clear_all_cache_now(obj: Optional[Atspi.Accessible] = None, reason: str = "") -> None:
+    def clear_all_cache_now(obj: Atspi.Accessible | None = None, reason: str = "") -> None:
         """Clears all cached information immediately."""
 
         AXUtilities._clear_all_dictionaries(reason)
@@ -153,7 +150,7 @@ class AXUtilities:
         return True
 
     @staticmethod
-    def find_active_window() -> Optional[Atspi.Accessible]:
+    def find_active_window() -> Atspi.Accessible | None:
         """Tries to locate the active window; may or may not succeed."""
 
         candidates = []
@@ -193,7 +190,7 @@ class AXUtilities:
             debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return filtered[0]
 
-        guess: Optional[Atspi.Accessible] = None
+        guess: Atspi.Accessible | None = None
         if filtered:
             tokens = ["AXUtilities: Still have multiple active windows:", filtered]
             debug.print_tokens(debug.LEVEL_INFO, tokens, True)
@@ -267,7 +264,7 @@ class AXUtilities:
         return AXObject.find_all_descendants(obj, is_match)
 
     @staticmethod
-    def get_default_button(obj: Atspi.Accessible) -> Optional[Atspi.Accessible]:
+    def get_default_button(obj: Atspi.Accessible) -> Atspi.Accessible | None:
         """Returns the default button descendant of obj"""
 
         result = None
@@ -279,7 +276,7 @@ class AXUtilities:
         return AXObject.find_descendant(obj, AXUtilitiesRole.is_default_button)
 
     @staticmethod
-    def get_focused_object(obj: Atspi.Accessible) -> Optional[Atspi.Accessible]:
+    def get_focused_object(obj: Atspi.Accessible) -> Atspi.Accessible | None:
         """Returns the focused descendant of obj"""
 
         result = None
@@ -291,7 +288,19 @@ class AXUtilities:
         return AXObject.find_descendant(obj, AXUtilitiesState.is_focused)
 
     @staticmethod
-    def get_status_bar(obj: Atspi.Accessible) -> Optional[Atspi.Accessible]:
+    def get_info_bar(obj: Atspi.Accessible) -> Atspi.Accessible | None:
+        """Returns the info bar descendant of obj"""
+
+        result = None
+        if AXObject.supports_collection(obj):
+            result = AXUtilitiesCollection.find_info_bar(obj)
+            if not AXUtilities.COMPARE_COLLECTION_PERFORMANCE:
+                return result
+
+        return AXObject.find_descendant(obj, AXUtilitiesRole.is_info_bar)
+
+    @staticmethod
+    def get_status_bar(obj: Atspi.Accessible) -> Atspi.Accessible | None:
         """Returns the status bar descendant of obj"""
 
         result = None
@@ -345,8 +354,9 @@ class AXUtilities:
                 if not AXUtilitiesRole.is_label(child) and child in labelled_by:
                     return True, "is labelled by non-label only child"
             set_roles = AXUtilitiesRole.get_set_container_roles()
-            if AXObject.find_ancestor(obj, lambda x: AXObject.get_role(x) in set_roles):
-                return True, "is in set container"
+            ancestor = AXObject.find_ancestor(obj, lambda x: AXObject.get_role(x) in set_roles)
+            if ancestor and AXObject.get_name(ancestor) == name:
+                return True, "is in set container with same name"
             return False, reason
 
         if AXUtilitiesRole.is_section(obj, role) or AXUtilitiesRole.is_document(obj, role):
@@ -650,32 +660,301 @@ class AXUtilities:
 
         return AXObject.get_attribute(obj, "explicit-name") == "true"
 
+    @staticmethod
+    def has_visible_caption(obj: Atspi.Accessible) -> bool:
+        """Returns True if obj has a visible caption."""
+
+        if not (AXUtilitiesRole.is_figure(obj) or AXObject.supports_table(obj)):
+            return False
+
+        labels = AXUtilitiesRelation.get_is_labelled_by(obj)
+        for label in labels:
+            if AXUtilitiesRole.is_caption(label) \
+               and AXUtilitiesState.is_showing(label) and AXUtilitiesState.is_visible(label):
+                return True
+
+        return False
+
     @staticmethod
     def get_displayed_label(obj: Atspi.Accessible) -> str:
         """Returns the displayed label of obj."""
 
-        if hash(obj) in AXUtilities.DISPLAYED_LABEL:
-            return AXUtilities.DISPLAYED_LABEL.get(hash(obj), "")
-
         labels = AXUtilitiesRelation.get_is_labelled_by(obj)
         strings = [AXObject.get_name(label) or AXText.get_all_text(label) for label in labels]
         result = " ".join(strings)
-        AXUtilities.DISPLAYED_LABEL[hash(obj)] = result
         return result
 
     @staticmethod
     def get_displayed_description(obj: Atspi.Accessible) -> str:
         """Returns the displayed description of obj."""
 
-        if hash(obj) in AXUtilities.DISPLAYED_DESCRIPTION:
-            return AXUtilities.DISPLAYED_DESCRIPTION.get(hash(obj), "")
-
         descriptions = AXUtilitiesRelation.get_is_described_by(obj)
         strings = [AXObject.get_name(desc) or AXText.get_all_text(desc) for desc in descriptions]
         result = " ".join(strings)
-        AXUtilities.DISPLAYED_DESCRIPTION[hash(obj)] = result
         return result
 
+    @staticmethod
+    def get_heading_level(obj: Atspi.Accessible) -> int:
+        """Returns the heading level of obj."""
+
+        if not AXUtilitiesRole.is_heading(obj):
+            return 0
+
+        use_cache = not AXUtilitiesState.is_editable(obj)
+        attrs = AXObject.get_attributes_dict(obj, use_cache)
+
+        try:
+            value = int(attrs.get("level", "0"))
+        except ValueError:
+            tokens = ["AXUtilities: Exception getting value for", obj, "(", attrs, ")"]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return 0
+
+        return value
+
+    @staticmethod
+    def get_nesting_level(obj: Atspi.Accessible) -> int:
+        """Returns the nesting level of obj."""
+
+        def pred(x: Atspi.Accessible) -> bool:
+            if AXUtilitiesRole.is_list_item(obj):
+                return AXUtilitiesRole.is_list(AXObject.get_parent(x))
+            return AXUtilitiesRole.have_same_role(obj, x)
+
+        ancestors = []
+        ancestor = AXObject.find_ancestor(obj, pred)
+        while ancestor:
+            ancestors.append(ancestor)
+            ancestor = AXObject.find_ancestor(ancestor, pred)
+
+        return len(ancestors)
+
+    @staticmethod
+    def get_next_object(obj: Atspi.Accessible) -> Atspi.Accessible | None:
+        """Returns the next object (depth first, unless there's a flows-to relation)"""
+
+        if not AXObject.is_valid(obj):
+            return None
+
+        targets = AXUtilitiesRelation.get_flows_to(obj)
+        for target in targets:
+            if not AXObject.is_dead(target):
+                return target
+
+        index = AXObject.get_index_in_parent(obj) + 1
+        parent = AXObject.get_parent(obj)
+        while parent and not 0 < index < AXObject.get_child_count(parent):
+            obj = parent
+            index = AXObject.get_index_in_parent(obj) + 1
+            parent = AXObject.get_parent(obj)
+
+        if parent is None:
+            return None
+
+        next_object = AXObject.get_child(parent, index)
+        if next_object == obj:
+            tokens = ["AXUtilities:", obj, "claims to be its own next object"]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return None
+
+        return next_object
+
+    @staticmethod
+    def get_previous_object(obj: Atspi.Accessible) -> Atspi.Accessible | None:
+        """Returns the previous object (depth first, unless there's a flows-from relation)"""
+
+        if not AXObject.is_valid(obj):
+            return None
+
+        targets = AXUtilitiesRelation.get_flows_from(obj)
+        for target in targets:
+            if not AXObject.is_dead(target):
+                return target
+
+        index = AXObject.get_index_in_parent(obj) - 1
+        parent = AXObject.get_parent(obj)
+        while parent and not 0 <= index < AXObject.get_child_count(parent) - 1:
+            obj = parent
+            index = AXObject.get_index_in_parent(obj) - 1
+            parent = AXObject.get_parent(obj)
+
+        if parent is None:
+            return None
+
+        previous_object = AXObject.get_child(parent, index)
+        if previous_object == obj:
+            tokens = ["AXUtilities:", obj, "claims to be its own previous object"]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return None
+
+        return previous_object
+
+    @staticmethod
+    def is_on_screen(
+        obj: Atspi.Accessible,
+        bounding_box: Atspi.Rect | None = None
+    ) -> bool:
+        """Returns true if obj should be treated as being on screen."""
+
+        AXObject.clear_cache(obj, False, "Updating to check if object is on screen.")
+
+        tokens = ["AXUtilities: Checking if", obj, "is showing and visible...."]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        if not (AXUtilitiesState.is_showing(obj) and AXUtilitiesState.is_visible(obj)):
+            tokens = ["AXUtilities:", obj, "is not showing and visible. Treating as off screen."]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return False
+
+        tokens = ["AXUtilities:", obj, "is showing and visible. Checking hidden..."]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        if AXUtilitiesState.is_hidden(obj):
+            tokens = ["AXUtilities:", obj, "is reports being hidden. Treating as off screen."]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return False
+
+        tokens = ["AXUtilities:", obj, "is not hidden. Checking size and rect..."]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        if AXComponent.has_no_size_or_invalid_rect(obj):
+            tokens = ["AXUtilities: Rect of", obj, "is unhelpful. Treating as on screen."]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return True
+
+        tokens = ["AXUtilities:", obj, "has size and a valid rect. Checking if off screen..."]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        if AXComponent.object_is_off_screen(obj):
+            tokens = ["AXUtilities:", obj, "is believed to be off screen."]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return False
+
+        tokens = ["AXUtilities:", obj, "is not off screen. Checking",
+                  bounding_box, "intersection..."]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        if bounding_box is not None and not AXComponent.object_intersects_rect(obj, bounding_box):
+            tokens = ["AXUtilities", obj, "not in", bounding_box, ". Treating as off screen."]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            return False
+
+        tokens = ["AXUtilities:", obj, "is believed to be on screen."]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+        return True
+
+    @staticmethod
+    def treat_as_leaf_node(obj: Atspi.Accessible) -> bool:
+        """Returns True if obj should be treated as a leaf node."""
+
+        if AXUtilitiesRole.children_are_presentational(obj):
+            # In GTK, the contents of the page tab descends from the page tab.
+            if AXUtilitiesRole.is_page_tab(obj):
+                return False
+            return True
+
+        role = AXObject.get_role(obj)
+        if AXUtilitiesRole.is_combo_box(obj, role):
+            return not AXUtilitiesState.is_expanded(obj)
+
+        if AXUtilitiesRole.is_menu(obj, role) and not AXUtilitiesRole.has_role_from_aria(obj):
+            return not AXUtilitiesState.is_expanded(obj)
+
+        if AXObject.get_name(obj):
+            return AXUtilitiesRole.is_link(obj, role) or AXUtilitiesRole.is_label(obj, role)
+
+        return False
+
+    @staticmethod
+    def _get_on_screen_objects(
+        root: Atspi.Accessible,
+        cancellation_event: threading.Event,
+        bounding_box: Atspi.Rect | None = None
+    ) -> list:
+
+        tokens = ["AXUtilities: Getting on-screen objects in", root, f"({hex(id(root))})"]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        if cancellation_event.is_set():
+            msg = "AXUtilities: Cancellation event set. Stopping search."
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return []
+
+        if not AXUtilities.is_on_screen(root, bounding_box):
+            return []
+
+        if AXUtilities.treat_as_leaf_node(root):
+            return [root]
+
+        if AXObject.supports_table(root) and AXObject.supports_selection(root):
+            return list(AXTable.iter_visible_cells(root))
+
+        objects = []
+        root_name = AXObject.get_name(root)
+        if root_name or AXObject.get_description(root) or AXText.has_presentable_text(root):
+            objects.append(root)
+
+        if bounding_box is None:
+            bounding_box = AXComponent.get_rect(root)
+
+        for i, child in enumerate(AXObject.iter_children(root)):
+            tokens = [f"AXUtilities: Child {i} is", child, f"({hex(id(child))})"]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            if cancellation_event.is_set():
+                msg = "AXUtilities: Cancellation event set. Stopping search."
+                debug.print_message(debug.LEVEL_INFO, msg, True)
+                break
+
+            children = AXUtilities._get_on_screen_objects(child, cancellation_event, bounding_box)
+            objects.extend(children)
+            if root_name and children and root in objects and root_name == AXObject.get_name(child):
+                objects.remove(root)
+
+        if objects:
+            return objects
+
+        if AXUtilitiesState.is_focusable(root) or AXObject.has_action(root, "click"):
+            return [root]
+
+        return []
+
+    @staticmethod
+    def get_on_screen_objects(
+        root: Atspi.Accessible,
+        bounding_box: Atspi.Rect | None = None,
+        timeout: float = 5.0
+    ) -> list:
+        """Returns a list of onscreen objects in the given root."""
+
+        result_queue: queue.Queue[list] = queue.Queue()
+        cancellation_event = threading.Event()
+
+        def _worker():
+            result = AXUtilities._get_on_screen_objects(root, cancellation_event, bounding_box)
+            if not cancellation_event.is_set():
+                result_queue.put(result)
+
+        worker_thread = threading.Thread(target=_worker)
+        worker_thread.start()
+
+        try:
+            result = result_queue.get(timeout=timeout)
+        except queue.Empty:
+            tokens = ["AXUtilities: get_on_screen_objects timed out.", root]
+            debug.print_tokens(debug.LEVEL_WARNING, tokens, True)
+            cancellation_event.set()
+            result = []
+
+            msg = "AXUtilities: Checking AT-SPI responsiveness...."
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            desktop = AXUtilitiesApplication.get_desktop()
+            tokens = ["AXUtilities: Desktop is", desktop]
+            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+
+        worker_thread.join()
+        tokens = [f"AXUtilities: {len(result)} onscreen objects found in", root]
+        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+        return result
 
 for method_name, method in inspect.getmembers(AXUtilitiesApplication, predicate=inspect.isfunction):
     setattr(AXUtilities, method_name, method)
