@@ -18,7 +18,6 @@
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
 
-# pylint: disable=broad-exception-caught
 # pylint: disable=wrong-import-position
 # pylint: disable=too-many-return-statements
 # pylint: disable=too-many-branches
@@ -41,13 +40,14 @@ import itertools
 import queue
 import threading
 import time
-from typing import Optional, TYPE_CHECKING
+from typing import TYPE_CHECKING
 
 import gi
 gi.require_version("Atspi", "2.0")
 from gi.repository import Atspi
 from gi.repository import GLib
 
+from . import braille
 from . import debug
 from . import focus_manager
 from . import input_event
@@ -68,7 +68,8 @@ class EventManager:
     PRIORITY_IMPORTANT = 2
     PRIORITY_HIGH = 3
     PRIORITY_NORMAL = 4
-    PRIORITY_LOW = 5
+    PRIORITY_LOWER = 5
+    PRIORITY_LOW = 6
 
     def __init__(self) -> None:
         debug.print_message(debug.LEVEL_INFO, "EVENT MANAGER: Initializing", True)
@@ -82,7 +83,7 @@ class EventManager:
         self._gidle_id: int = 0
         self._gidle_lock = threading.Lock()
         self._listener: Atspi.EventListener = Atspi.EventListener.new(self._enqueue_object_event)
-        self._event_history: dict[str, tuple[Optional[int], float]] = {}
+        self._event_history: dict[str, tuple[int | None, float]] = {}
         debug.print_message(debug.LEVEL_INFO, "Event manager initialized", True)
 
     def activate(self) -> None:
@@ -121,6 +122,7 @@ class EventManager:
         self._paused = pause
         if clear_queue:
             self._event_queue = queue.PriorityQueue(0)
+        input_event_manager.get_manager().pause_key_watcher(pause, reason)
 
     def _get_priority(self, event: Atspi.Event) -> int:
         """Returns the priority associated with event."""
@@ -135,6 +137,17 @@ class EventManager:
             priority = EventManager.PRIORITY_HIGH
         elif event_type.startswith("object:active-descendant-changed"):
             priority = EventManager.PRIORITY_HIGH
+        elif event_type.startswith("object:announcement"):
+            if event.detail1 == Atspi.Live.ASSERTIVE:
+                priority = EventManager.PRIORITY_IMPORTANT
+            elif event.detail1 == Atspi.Live.POLITE:
+                priority = EventManager.PRIORITY_HIGH
+            else:
+                priority = EventManager.PRIORITY_NORMAL
+        elif event_type.startswith("object:state-changed:invalid-entry"):
+            # Setting this to lower ensures we present the state and/or text changes that triggered
+            # the invalid state prior to presenting the invalid state.
+            priority = EventManager.PRIORITY_LOWER
         elif event_type.startswith("object:children-changed"):
             priority = EventManager.PRIORITY_LOW
         else:
@@ -144,7 +157,7 @@ class EventManager:
         debug.print_tokens(debug.LEVEL_INFO, tokens, True)
         return priority
 
-    def _is_obsoleted_by(self, event: Atspi.Event) -> Optional[Atspi.Event]:
+    def _is_obsoleted_by(self, event: Atspi.Event) -> Atspi.Event | None:
         """Returns the event which renders this one no longer worthy of being processed."""
 
         def is_same(x):
@@ -198,7 +211,7 @@ class EventManager:
         with self._event_queue.mutex:
             try:
                 events = list(reversed(self._event_queue.queue))
-            except Exception as error:
+            except queue.Empty as error:
                 msg = f"EVENT MANAGER: Exception in _isObsoletedBy: {error}"
                 debug.print_message(debug.LEVEL_INFO, msg, True)
                 events = []
@@ -242,7 +255,7 @@ class EventManager:
             return True
 
         event_type = event.type
-        if event_type.startswith('window') or event_type.startswith('mouse:button'):
+        if event_type.startswith(('window', 'mouse:button')):
             return False
 
         # gnome-shell fires "focused" events spuriously after the Alt+Tab switcher
@@ -529,8 +542,8 @@ class EventManager:
 
         if script_manager.get_manager().get_active_script() is None:
             default_script = script_manager.get_manager().get_default_script()
-            script_manager.get_manager().set_active_script(default_script, 'No focus')
-            default_script.idleMessage()
+            script_manager.get_manager().set_active_script(default_script, "No focus")
+            braille.disableBraille()
 
         return False
 
@@ -560,14 +573,11 @@ class EventManager:
                     GLib.timeout_add(2500, self._on_no_focus)
                     self._gidle_id = 0
                     rerun = False  # destroy and don't call again
-
         except queue.Empty:
             msg = 'EVENT MANAGER: Attempted dequeue, but the event queue is empty'
             debug.print_message(debug.LEVEL_SEVERE, msg, True)
             self._gidle_id = 0
             rerun = False # destroy and don't call again
-        except Exception:
-            debug.print_exception(debug.LEVEL_SEVERE)
 
         return rerun
 
@@ -602,7 +612,11 @@ class EventManager:
 
         self._script_listener_counts[event_type] -= 1
         if self._script_listener_counts[event_type] == 0:
-            self._listener.deregister(event_type)
+            try:
+                self._listener.deregister(event_type)
+            except GLib.GError as error:
+                msg = f"EVENT MANAGER: Exception deregistering listener for {event_type}: {error}"
+                debug.print_message(debug.LEVEL_INFO, msg, True)
             del self._script_listener_counts[event_type]
 
     def register_script_listeners(self, script: default.Script) -> None:
@@ -635,8 +649,8 @@ class EventManager:
 
     @staticmethod
     def _get_script_for_event(
-        event: Atspi.Event, active_script: Optional[default.Script] = None
-    ) -> Optional[default.Script]:
+        event: Atspi.Event, active_script: default.Script | None = None
+    ) -> default.Script | None:
         """Returns the script associated with event."""
 
         if event.source == focus_manager.get_manager().get_locus_of_focus():
@@ -669,7 +683,7 @@ class EventManager:
         return script
 
     def _is_activatable_event(
-        self, event: Atspi.Event, script: Optional[default.Script] = None
+        self, event: Atspi.Event, script: default.Script | None = None
     ) -> tuple[bool, str]:
         """Determines if event should cause us to change the active script."""
 
@@ -821,13 +835,12 @@ class EventManager:
                 if event.type.startswith(key):
                     listener = value
                     break
-
-        try:
-            listener(event)
-        except Exception as error:
-            msg = f"EVENT MANAGER: Exception processing {event.type}: {error}"
+        if listener is None:
+            msg = f"EVENT MANAGER: No listener for event type {event.type}"
             debug.print_message(debug.LEVEL_INFO, msg, True)
-            debug.print_exception(debug.LEVEL_INFO)
+            return
+
+        listener(event)
 
 _manager: EventManager = EventManager()
 
