@@ -38,7 +38,8 @@ __license__   = "LGPL"
 import inspect
 import math
 import time
-from typing import Any, Callable, Optional, TYPE_CHECKING, cast
+import unicodedata
+from typing import Callable, TYPE_CHECKING
 
 import gi
 gi.require_version("Atspi", "2.0")
@@ -63,6 +64,7 @@ if TYPE_CHECKING:
 KEYBOARD_EVENT     = "keyboard"
 BRAILLE_EVENT      = "braille"
 MOUSE_BUTTON_EVENT = "mouse:button"
+REMOTE_CONTROLLER_EVENT = "remote controller"
 
 class InputEvent:
     """Provides support for handling input events."""
@@ -119,16 +121,16 @@ class KeyboardEvent(InputEvent):
         )
         self.hw_code: int = keycode
         self._text: str = text
-        self.modifiers: int = modifiers & Gdk.ModifierType.MODIFIER_MASK
+        self.modifiers: int = modifiers & Gdk.ModifierType.MODIFIER_MASK # pylint: disable=no-member
         if modifiers & (1 << Atspi.ModifierType.NUMLOCK):
             self.modifiers |= (1 << Atspi.ModifierType.NUMLOCK)
-        self.keyval_name: str = Gdk.keyval_name(keysym)
+        self.keyval_name: str = Gdk.keyval_name(keysym) or ""
         self.timestamp: float = time.time()
-        self._script: Optional[default.Script] = None
-        self._window: Optional[Atspi.Accessible] = None
-        self._obj: Optional[Atspi.Accessible] = None
-        self._handler: Optional[InputEventHandler] = None
-        self._consumer: Optional[Callable[..., bool]] = None
+        self._script: default.Script | None = None
+        self._window: Atspi.Accessible | None = None
+        self._obj: Atspi.Accessible | None = None
+        self._handler: InputEventHandler | None = None
+        self._consumer: Callable[..., bool] | None = None
         self._is_kp_with_numlock: bool = False
 
         # Some implementors don't include numlock in the modifiers. Unfortunately,
@@ -157,7 +159,7 @@ class KeyboardEvent(InputEvent):
         return False
 
     def __str__(self) -> str:
-        if self._should_obscure():
+        if self.should_obscure():
             keyid = hw_code = modifiers = text = keyval_name = "*"
         else:
             keyid = str(self.id)
@@ -178,7 +180,7 @@ class KeyboardEvent(InputEvent):
     def as_single_line_string(self) -> str:
         """Returns a single-line string representation of this event."""
 
-        if self._should_obscure():
+        if self.should_obscure():
             return "(obscured)"
 
         return (
@@ -186,7 +188,17 @@ class KeyboardEvent(InputEvent):
             f"{self.type.value_nick}"
         )
 
-    def _should_obscure(self) -> bool:
+    def is_alt_control_or_orca_modified(self) -> bool:
+        """Return True if this key is Alt, Control, or Orca modified."""
+
+        if self.modifiers & keybindings.CTRL_MODIFIER_MASK \
+           or self.modifiers & keybindings.ALT_MODIFIER_MASK \
+           or self.modifiers & keybindings.ORCA_MODIFIER_MASK:
+            return True
+
+        return False
+
+    def should_obscure(self) -> bool:
         """Returns True if we should obscure the details of this event."""
 
         if not AXUtilities.is_password_text(self._obj):
@@ -195,9 +207,7 @@ class KeyboardEvent(InputEvent):
         if not self.is_printable_key():
             return False
 
-        if self.modifiers & keybindings.CTRL_MODIFIER_MASK \
-           or self.modifiers & keybindings.ALT_MODIFIER_MASK \
-           or self.modifiers & keybindings.ORCA_MODIFIER_MASK:
+        if self.is_alt_control_or_orca_modified():
             return False
 
         return True
@@ -222,6 +232,7 @@ class KeyboardEvent(InputEvent):
             Gdk.KEY_BackSpace,
             Gdk.KEY_Delete,
             Gdk.KEY_Escape,
+            Gdk.KEY_KP_Enter,
             Gdk.KEY_Page_Down,
             Gdk.KEY_Page_Up,
             Gdk.KEY_Return,
@@ -295,7 +306,18 @@ class KeyboardEvent(InputEvent):
             Gdk.KEY_dead_u,
             Gdk.KEY_dead_voiced_sound,
         ]
-        return self.id in keys
+
+        if self.id in keys:
+            return True
+
+        name = self.get_key_name()
+        if len(name) == 1:
+            category = unicodedata.category(name)
+            # Mn = Mark, nonspacing; Mc = Mark, spacing combining; Me = Mark, enclosing
+            if category in ("Mn", "Mc", "Me"):
+                return True
+
+        return False
 
     def is_function_key(self) -> bool:
         """Return True if this is a function key."""
@@ -360,6 +382,16 @@ class KeyboardEvent(InputEvent):
             Gdk.KEY_7,
             Gdk.KEY_8,
             Gdk.KEY_9,
+            Gdk.KEY_KP_0,
+            Gdk.KEY_KP_1,
+            Gdk.KEY_KP_2,
+            Gdk.KEY_KP_3,
+            Gdk.KEY_KP_4,
+            Gdk.KEY_KP_5,
+            Gdk.KEY_KP_6,
+            Gdk.KEY_KP_7,
+            Gdk.KEY_KP_8,
+            Gdk.KEY_KP_9,
         ]
         return self.id in keys
 
@@ -434,6 +466,11 @@ class KeyboardEvent(InputEvent):
             Gdk.KEY_guillemotleft,
             Gdk.KEY_guillemotright,
             Gdk.KEY_hyphen,
+            Gdk.KEY_KP_Decimal,
+            Gdk.KEY_KP_Add,
+            Gdk.KEY_KP_Divide,
+            Gdk.KEY_KP_Multiply,
+            Gdk.KEY_KP_Subtract,
             Gdk.KEY_less,
             Gdk.KEY_macron,
             Gdk.KEY_minus,
@@ -467,40 +504,7 @@ class KeyboardEvent(InputEvent):
 
         return self.id == Gdk.KEY_space
 
-    def is_character_echoable(self) -> bool:
-        """Returns True if the script will echo this event as part of character echo."""
-
-        if not settings.enableEchoByCharacter:
-            msg = "KEYBOARD EVENT: Not character echoable, setting disabled."
-            debug.print_message(debug.LEVEL_INFO, msg, True)
-            return False
-
-        if not self.is_printable_key():
-            msg = "KEYBOARD EVENT: Not character echoable, is not printable key."
-            debug.print_message(debug.LEVEL_INFO, msg, True)
-            return False
-
-        # TODO - JD: What is this check handling specifically?
-        if self.modifiers & keybindings.ORCA_CTRL_MODIFIER_MASK:
-            msg = "KEYBOARD EVENT: Not character echoable due to modifier mask."
-            debug.print_message(debug.LEVEL_INFO, msg, True)
-            return False
-
-        if AXUtilities.is_password_text(self._obj):
-            msg = "KEYBOARD EVENT: Not character echoable, is password text."
-            debug.print_message(debug.LEVEL_INFO, msg, True)
-            return False
-
-        if AXUtilities.is_editable(self._obj) or AXUtilities.is_terminal(self._obj):
-            msg = "KEYBOARD EVENT: Character echoable, is editable or terminal."
-            debug.print_message(debug.LEVEL_INFO, msg, True)
-            return True
-
-        msg = "KEYBOARD EVENT: Not character echoable, no reason to echo."
-        debug.print_message(debug.LEVEL_INFO, msg, True)
-        return False
-
-    def get_locking_state(self) -> Optional[bool]:
+    def get_locking_state(self) -> bool | None:
         """Returns True if the event locked a locking key, False if the event unlocked it, and None
         if not a locking key."""
 
@@ -548,12 +552,12 @@ class KeyboardEvent(InputEvent):
 
         return self.keyval_name
 
-    def get_object(self) -> Optional[Atspi.Accessible]:
+    def get_object(self) -> Atspi.Accessible | None:
         """Returns the object believed to be associated with this key event."""
 
         return self._obj
 
-    def set_object(self, obj: Optional[Atspi.Accessible]) -> None:
+    def set_object(self, obj: Atspi.Accessible | None) -> None:
         """Sets the object believed to be associated with this key event."""
 
         module_name = inspect.getmodulename(inspect.stack()[1].filename)
@@ -562,12 +566,12 @@ class KeyboardEvent(InputEvent):
 
         self._obj = obj
 
-    def get_window(self) -> Optional[Atspi.Accessible]:
+    def get_window(self) -> Atspi.Accessible | None:
         """Returns the window believed to be associated with this key event."""
 
         return self._window
 
-    def set_window(self, window: Optional[Atspi.Accessible]) -> None:
+    def set_window(self, window: Atspi.Accessible | None) -> None:
         """Sets the window believed to be associated with this key event."""
 
         module_name = inspect.getmodulename(inspect.stack()[1].filename)
@@ -576,12 +580,12 @@ class KeyboardEvent(InputEvent):
 
         self._window = window
 
-    def get_script(self) -> Optional[default.Script]:
+    def get_script(self) -> default.Script | None:
         """Returns the script believed to be associated with this key event."""
 
         return self._script
 
-    def set_script(self, script: Optional[default.Script]) -> None:
+    def set_script(self, script: default.Script | None) -> None:
         """Sets the script believed to be associated with this key event."""
 
         module_name = inspect.getmodulename(inspect.stack()[1].filename)
@@ -590,12 +594,12 @@ class KeyboardEvent(InputEvent):
 
         self._script = script
 
-    def get_handler(self) -> Optional[InputEventHandler]:
+    def get_handler(self) -> InputEventHandler | None:
         """Returns the handler associated with this key event."""
 
         return self._handler
 
-    def _get_user_handler(self) -> Optional[InputEventHandler]:
+    def _get_user_handler(self) -> InputEventHandler | None:
         # TODO - JD: This should go away once plugin support is in place.
         try:
             bindings = settings.keyBindingsMap.get(self._script.__module__)
@@ -617,64 +621,24 @@ class KeyboardEvent(InputEvent):
 
         return handler
 
-    def _present(self) -> bool:
+    def _present(self) -> None:
         if not self._script:
-            return False
+            return
 
         if self.is_pressed_key():
-            self._script.presentationInterrupt()
+            self._script.interrupt_presentation()
 
         if self._script.get_learn_mode_presenter().is_active():
-            return False
+            return
 
-        return self._script.presentKeyboardEvent(self)
-
-    # pylint:disable=too-many-branches
-    # pylint:disable=too-many-return-statements
-    def should_echo(self) -> bool:
-        """Returns True if this input event should be echoed."""
-
-        if not (self.is_pressed_key() or AXUtilities.is_terminal(self._obj)):
-            return False
-
-        if self.is_locking_key():
-            if settings.presentLockingKeys is None:
-                return not settings.onlySpeakDisplayedText
-            return settings.presentLockingKeys
-
-        if not settings.enableKeyEcho:
-            return False
-
-        if self.is_navigation_key():
-            return settings.enableNavigationKeys
-        if self.is_action_key():
-            return settings.enableActionKeys
-        if self.is_modifier_key():
-            return settings.enableModifierKeys
-        if self.is_function_key():
-            return settings.enableFunctionKeys
-        if self.is_diacritical_key():
-            if settings.enableDiacriticalKeys is None:
-                return not settings.onlySpeakDisplayedText
-            return settings.enableDiacriticalKeys
-        if self.is_alphabetic_key():
-            return settings.enableAlphabeticKeys or settings.enableEchoByCharacter
-        if self.is_numeric_key():
-            return settings.enableNumericKeys or settings.enableEchoByCharacter
-        if self.is_punctuation_key():
-            return settings.enablePunctuationKeys or settings.enableEchoByCharacter
-        if self.is_space():
-            return settings.enableSpace or settings.enableEchoByCharacter
-
-        return False
-    # pylint:enable=too-many-branches
-    # pylint:enable=too-many-return-statements
+        self._script.present_keyboard_event(self)
 
     def process(self) -> None:
         """Processes this input event."""
 
         start_time = time.time()
-        if not self._should_obscure():
+        should_obscure = self.should_obscure()
+        if not should_obscure:
             data = f"'{self.keyval_name}' ({self.hw_code})"
         else:
             data = "(obscured)"
@@ -696,12 +660,13 @@ class KeyboardEvent(InputEvent):
         if self._script:
             self._handler = self._get_user_handler() \
                 or self._script.key_bindings.get_input_handler(self)
-            tokens = ["HANDLER:", cast(Any, self._handler)]
-            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
+            if not should_obscure:
+                tokens = ["HANDLER:", str(self._handler)]
+                debug.print_tokens(debug.LEVEL_INFO, tokens, True)
 
             if self._script.get_learn_mode_presenter().is_active():
                 self._consumer = self._script.get_learn_mode_presenter().handle_event
-                tokens = ["CONSUMER:", cast(Any, self._consumer)]
+                tokens = ["CONSUMER:", str(self._consumer)]
                 debug.print_tokens(debug.LEVEL_INFO, tokens, True)
 
         if self.is_orca_modifier() and self._click_count == 2:
@@ -754,12 +719,12 @@ class BrailleEvent(InputEvent):
     def __init__(self, event: dict) -> None:
         super().__init__(BRAILLE_EVENT)
         self.event: dict = event
-        self._script: Optional[default.Script] = script_manager.get_manager().get_active_script()
+        self._script: default.Script | None = script_manager.get_manager().get_active_script()
 
     def __str__(self) -> str:
         return f"{self.type.upper()} {self.event}"
 
-    def get_handler(self) -> Optional[InputEventHandler]:
+    def get_handler(self) -> InputEventHandler | None:
         """Returns the handler associated with this event."""
 
         try:
@@ -770,7 +735,7 @@ class BrailleEvent(InputEvent):
             return None
 
         command: str = self.event["command"]
-        user_bindings: Optional[dict] = None
+        user_bindings: dict | None = None
         user_bindings_map: dict = settings.brailleBindingsMap
         if self._script.name in user_bindings_map:
             user_bindings = user_bindings_map[self._script.name]
@@ -778,7 +743,7 @@ class BrailleEvent(InputEvent):
             user_bindings = user_bindings_map.get("default")
 
         if user_bindings and command in user_bindings:
-            handler: InputEventHandler = user_bindings[command]
+            handler: InputEventHandler | None = user_bindings[command]
             tokens = [f"BRAILLE EVENT: User handler for command {command} is", handler]
             debug.print_tokens(debug.LEVEL_INFO, tokens, True)
             return handler
@@ -825,6 +790,8 @@ class BrailleEvent(InputEvent):
 class MouseButtonEvent(InputEvent):
     """Provides support for handling mouse button events."""
 
+    # TODO - JD: Remove this and the validation logic once we have a fix for
+    # https://gitlab.gnome.org/GNOME/at-spi2-core/-/issues/194.
     try:
         display = Gdk.Display.get_default()
         seat = Gdk.Display.get_default_seat(display)
@@ -871,6 +838,12 @@ class MouseButtonEvent(InputEvent):
         debug.print_message(debug.LEVEL_INFO, msg, True)
         self.x, self.y = x, y
 
+class RemoteControllerEvent(InputEvent):
+    """A simple input event whose main purpose is identification of the origin."""
+
+    def __init__(self):
+        super().__init__(REMOTE_CONTROLLER_EVENT)
+
 class InputEventHandler:
     """A handler for an input event."""
 
