@@ -21,6 +21,7 @@
 
 # pylint: disable=wrong-import-position
 # pylint: disable=too-many-public-methods
+# pylint: disable=too-many-lines
 
 """Provides utilities for managing input events."""
 
@@ -38,9 +39,7 @@ from typing import TYPE_CHECKING
 
 import gi
 gi.require_version("Atspi", "2.0")
-gi.require_version("Gdk", "3.0")
 from gi.repository import Atspi
-from gi.repository import Gdk
 
 from . import debug
 from . import focus_manager
@@ -60,6 +59,10 @@ class InputEventManager:
         self._last_input_event: input_event.InputEvent | None = None
         self._last_non_modifier_key_event: input_event.KeyboardEvent | None = None
         self._device: Atspi.Device | None = None
+        self._mapped_keycodes: list[int] = []
+        self._mapped_keysyms: list[int] = []
+        self._grabbed_bindings: dict[int, keybindings.KeyBinding] = {}
+        self._paused: bool = False
 
     def start_key_watcher(self) -> None:
         """Starts the watcher for keyboard input events."""
@@ -79,6 +82,22 @@ class InputEventManager:
         debug.print_message(debug.LEVEL_INFO, msg, True)
         self._device = None
 
+    def pause_key_watcher(self, pause: bool = True, reason: str = "") -> None:
+        """Pauses processing of keyboard input events."""
+
+        msg = f"INPUT EVENT MANAGER: Pause queueing: {pause}. {reason}"
+        debug.print_message(debug.LEVEL_INFO, msg, True)
+        self._paused = pause
+
+    def check_grabbed_bindings(self) -> None:
+        """Checks the grabbed key bindings."""
+
+        msg = f"INPUT EVENT MANAGER: {len(self._grabbed_bindings)} grabbed key bindings."
+        debug.print_message(debug.LEVEL_INFO, msg, True)
+        for grab_id, binding in self._grabbed_bindings.items():
+            msg = f"INPUT EVENT MANAGER: {grab_id} for: {binding}"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+
     def add_grabs_for_keybinding(self, binding: keybindings.KeyBinding) -> list[int]:
         """Adds grabs for binding if it is enabled, returns grab IDs."""
 
@@ -97,7 +116,9 @@ class InputEventManager:
 
         grab_ids = []
         for kd in binding.key_definitions():
-            grab_ids.append(self._device.add_key_grab(kd, None))
+            grab_id = self._device.add_key_grab(kd, None)
+            grab_ids.append(grab_id)
+            self._grabbed_bindings[grab_id] = binding
 
         return grab_ids
 
@@ -117,6 +138,10 @@ class InputEventManager:
 
         for grab_id in grab_ids:
             self._device.remove_key_grab(grab_id)
+            removed = self._grabbed_bindings.pop(grab_id, None)
+            if removed is None:
+                msg = f"INPUT EVENT MANAGER: No key binding for grab id {grab_id}"
+                debug.print_message(debug.LEVEL_INFO, msg, True)
 
     def map_keycode_to_modifier(self, keycode: int) -> int:
         """Maps keycode as a modifier, returns the newly-mapped modifier."""
@@ -126,6 +151,7 @@ class InputEventManager:
             debug.print_message(debug.LEVEL_INFO, msg, True)
             return 0
 
+        self._mapped_keycodes.append(keycode)
         return self._device.map_modifier(keycode)
 
     def map_keysym_to_modifier(self, keysym: int) -> int:
@@ -136,8 +162,26 @@ class InputEventManager:
             debug.print_message(debug.LEVEL_INFO, msg, True)
             return 0
 
+        self._mapped_keysyms.append(keysym)
         return self._device.map_keysym_modifier(keysym)
 
+    def unmap_all_modifiers(self) -> None:
+        """Unmaps all previously mapped modifiers."""
+
+        if self._device is None:
+            msg = "INPUT EVENT MANAGER: No device to unmap all modifiers from"
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return
+
+        for keycode in self._mapped_keycodes:
+            self._device.unmap_modifier(keycode)
+
+        for keysym in self._mapped_keysyms:
+            self._device.unmap_keysym_modifier(keysym)
+
+        self._mapped_keycodes.clear()
+        self._mapped_keysyms.clear()
+
     def add_grab_for_modifier(self, modifier: str, keysym: int, keycode: int) -> int:
         """Adds grab for modifier, returns grab id."""
 
@@ -202,6 +246,14 @@ class InputEventManager:
         mouse_event.set_click_count(self._determine_mouse_event_click_count(mouse_event))
         self._last_input_event = mouse_event
 
+    def process_remote_controller_event(self, event: input_event.RemoteControllerEvent) -> None:
+        """Processes this RemoteController event."""
+
+        # TODO - JD: It probably makes sense to process remote controller events here rather
+        # than just updating state.
+        self._last_input_event = event
+        self._last_non_modifier_key_event = None
+
     # pylint: disable=too-many-arguments
     # pylint: disable=too-many-positional-arguments
     def process_keyboard_event(
@@ -215,6 +267,11 @@ class InputEventManager:
     ) -> bool:
         """Processes this Atspi keyboard event."""
 
+        if self._paused:
+            msg = "INPUT EVENT MANAGER: Keyboard event processing is paused."
+            debug.print_message(debug.LEVEL_INFO, msg, True)
+            return False
+
         event = input_event.KeyboardEvent(pressed, keycode, keysym, modifiers, text)
         if event in [self._last_input_event, self._last_non_modifier_key_event]:
             msg = "INPUT EVENT MANAGER: Received duplicate event."
@@ -344,7 +401,13 @@ class InputEventManager:
         return result
 
     def last_event_equals_or_is_release_for_event(self, event):
-        """Returns True if the last non-modifier event equals, or is the release for, event."""
+        """Returns True if the last event equals the provided event, or is the release for it."""
+
+        if self._last_input_event is event:
+            return True
+
+        if not self.last_event_was_keyboard():
+            return False
 
         if self._last_non_modifier_key_event is None:
             return False
@@ -365,25 +428,6 @@ class InputEventManager:
 
         return self._last_non_modifier_key_event.keyval_name, self._last_input_event.modifiers
 
-    def _last_keycode_and_modifiers(self):
-        """Returns the last keycode and modifiers"""
-
-        if self._last_non_modifier_key_event is None:
-            return 0, 0
-
-        if not isinstance(self._last_input_event, input_event.KeyboardEvent):
-            return 0, 0
-
-        return self._last_non_modifier_key_event.hw_code, self._last_input_event.modifiers
-
-    @staticmethod
-    def _all_names_for_key_code(keycode):
-        """Returns all the possible names associated with keycode."""
-
-        keymap = Gdk.Keymap.get_default()
-        entries = keymap.get_entries_for_keycode(keycode)[-1]
-        return list(map(Gdk.keyval_name, set(entries)))
-
     def last_event_was_command(self):
         """Returns True if the last event is believed to be a command."""
 
@@ -780,11 +824,10 @@ class InputEventManager:
     def last_event_was_delete(self):
         """Returns True if the last event is believed to be delete."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "Delete" in keynames or "KP_Delete" in keynames:
+        string, mods = self._last_key_and_modifiers()
+        if string in ["Delete", "KP_Delete"]:
             rv = True
-        elif "d" in keynames:
+        elif string.lower() == "d":
             rv = bool(mods & 1 << Atspi.ModifierType.CONTROL)
         else:
             rv = False
@@ -797,9 +840,8 @@ class InputEventManager:
     def last_event_was_cut(self):
         """Returns True if the last event is believed to be the cut command."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "x" not in keynames:
+        string, mods = self._last_key_and_modifiers()
+        if string.lower() != "x":
             return False
 
         if mods & 1 << Atspi.ModifierType.CONTROL and not mods & 1 << Atspi.ModifierType.SHIFT:
@@ -811,9 +853,8 @@ class InputEventManager:
     def last_event_was_copy(self):
         """Returns True if the last event is believed to be the copy command."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "c" not in keynames or not mods & 1 << Atspi.ModifierType.CONTROL:
+        string, mods = self._last_key_and_modifiers()
+        if string.lower() != "c" or not mods & 1 << Atspi.ModifierType.CONTROL:
             rv = False
         elif AXUtilities.is_terminal(self._last_input_event.get_object()):
             rv = mods & 1 << Atspi.ModifierType.SHIFT
@@ -828,9 +869,8 @@ class InputEventManager:
     def last_event_was_paste(self):
         """Returns True if the last event is believed to be the paste command."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "v" not in keynames or not mods & 1 << Atspi.ModifierType.CONTROL:
+        string, mods = self._last_key_and_modifiers()
+        if string.lower() != "v" or not mods & 1 << Atspi.ModifierType.CONTROL:
             rv = False
         elif AXUtilities.is_terminal(self._last_input_event.get_object()):
             rv = mods & 1 << Atspi.ModifierType.SHIFT
@@ -845,9 +885,8 @@ class InputEventManager:
     def last_event_was_undo(self):
         """Returns True if the last event is believed to be the undo command."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "z" not in keynames:
+        string, mods = self._last_key_and_modifiers()
+        if string.lower() != "z":
             return False
         if mods & 1 << Atspi.ModifierType.CONTROL and not mods & 1 << Atspi.ModifierType.SHIFT:
             msg = "INPUT EVENT MANAGER: Last event was undo"
@@ -858,11 +897,10 @@ class InputEventManager:
     def last_event_was_redo(self):
         """Returns True if the last event is believed to be the redo command."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "z" in keynames:
+        string, mods = self._last_key_and_modifiers()
+        if string.lower() == "z":
             rv = mods & 1 << Atspi.ModifierType.CONTROL and mods & 1 << Atspi.ModifierType.SHIFT
-        elif "y" in keynames:
+        elif string.lower() == "y":
             # LibreOffice
             rv = mods & 1 << Atspi.ModifierType.CONTROL \
                 and not mods & 1 << Atspi.ModifierType.SHIFT
@@ -877,9 +915,8 @@ class InputEventManager:
     def last_event_was_select_all(self):
         """Returns True if the last event is believed to be the select all command."""
 
-        keycode, mods = self._last_keycode_and_modifiers()
-        keynames = self._all_names_for_key_code(keycode)
-        if "a" not in keynames:
+        string, mods = self._last_key_and_modifiers()
+        if string.lower() != "a":
             return False
 
         if (mods & 1 << Atspi.ModifierType.CONTROL and not mods & 1 << Atspi.ModifierType.SHIFT):
