@@ -1,4 +1,5 @@
+"""Custom script for GTK."""
+
 # https://gitlab.gnome.org/GNOME/orca/-/issues/358
 # ruff: noqa: F401
 from .script import Script
-from .script_utilities import Utilities
